from django import forms
from django.core.exceptions import ValidationError
from django.utils.translation import gettext_lazy as _

from servala.core.models import ControlPlane


class ControlPlaneAdminForm(forms.ModelForm):
    certificate_authority_data = forms.CharField(
        widget=forms.Textarea,
        required=False,
        help_text=_("Certificate authority data for the Kubernetes API"),
    )
    server = forms.URLField(
        required=False,
        help_text=_("Server URL for the Kubernetes API"),
    )
    token = forms.CharField(
        widget=forms.Textarea,
        required=False,
        help_text=_("Authentication token for the Kubernetes API"),
    )

    class Meta:
        model = ControlPlane
        fields = "__all__"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        # If we have existing api_credentials, populate the individual fields
        if self.instance.pk and self.instance.api_credentials:
            creds = self.instance.api_credentials
            self.fields["certificate_authority_data"].initial = creds.get(
                "certificate-authority-data", ""
            )
            self.fields["server"].initial = creds.get("server", "")
            self.fields["token"].initial = creds.get("token", "")

    def clean(self):
        cleaned_data = super().clean()

        # Get the three credential fields
        ca_data = cleaned_data.get("certificate_authority_data")
        server = cleaned_data.get("server")
        token = cleaned_data.get("token")

        # Check if any of the fields are filled
        has_ca = bool(ca_data)
        has_server = bool(server)
        has_token = bool(token)

        # If some but not all fields are filled, raise validation error
        if (has_ca or has_server or has_token) and not (
            has_ca and has_server and has_token
        ):
            raise ValidationError(
                _(
                    "All credential fields (certificate authority data, server, and token) must be provided together or left empty."
                )
            )

        # If all fields are filled, create the api_credentials JSON
        if has_ca and has_server and has_token:
            cleaned_data["api_credentials"] = {
                "certificate-authority-data": ca_data,
                "server": server,
                "token": token,
            }
        else:
            cleaned_data["api_credentials"] = {}

        return cleaned_data

    def save(self, *args, **kwargs):
        self.instance.api_credentials = self.cleaned_data["api_credentials"]
        return super().save(*args, **kwargs)
